;+=----------------------------------------------------------------------------
; Name   : example_01.pro
; Desc   : Example program 1, covering the use of mrImageSetDicom
; Author : (c) Oliver Thilmann, Aug 2004
;=-----------------------------------------------------------------------------
pro example_01

	mrlib_init

	; mrImageSetDicom is an object to simplify data loading and handling, esp.
	; when it comes to larger data sets, e.g. multi slice images or time sequences
	; of multi slice images.

	; -------------------------------------------------
	;    Example how to load a single file
	; -------------------------------------------------

		; Initialize mrDicom - see the documentation

	mrdicom_setUIDs, 'mrimg', '0.0.00.0.0000.0.0'


		; instanciate the object with space for just one image

	dataSet  = obj_new( 'mrImageSetDicom', 1)

		; present file selector box to the user

	file = dialog_pickfile( path='c:\', get_path = path )

		; and load the selected image to index '0'

	err = dataSet->load( file, 0, /rotate )

		; check for errors

	if( err eq 0 ) then begin
		x = dialog_message(["Error loading single image", dataSet->errMsg()])
		return
	end

		; get the image data

	imgdata = dataSet->ImgData()

		; imgdata is a pointer to a matrix of pointers - since we loaded one
		; single image, we use the very first element (there's only one anyway)

	img = (*imgdata)[0]   ; who remembers operator precedences? :)

		; img now is a pointer to the image array - display the image

	loadct, 0
	tvscl, *img

		; destroying the mrdata object will also destroy the image data!

	obj_destroy, dataSet

	; -------------------------------------------------
	;    Example how to load a multislice sequence
	; -------------------------------------------------

	dataSet = obj_new('mrImageSetDicom')

	file = dialog_pickfile( path=path, get_path = path )

		; Load all images from the directory that have the same series number
		; as the selected file
		; The progressbar option makes mrImageSetDicom display a progressbar
		; while scanning all files in the directory

		; After loading, the images are sorted by image number
		; The image numbers must be consecutive (though they need not
		; start with '1') - otherwise the operation results in an error

	err  = dataSet->loadSequence( file, /progressbar, /rotate )

		; error checking

	if( err eq 0 ) then begin
		x = dialog_message(['Error loading image sequence:', dataSet->errMsg()])
		return
	end

		; print some info about the loaded data

	print, 'Results from sequence load'
	print, 'Number of slices: ', string( dataSet->nSlices() )
	print, 'Series number   : ', string( dataSet->seriesNumber() )

		; this time we display the last image in the sequence

	nof_imgs = dataSet->nSlices()

	img = (*(dataSet->imgData()))[nof_imgs-1]

		; display the image and destroy the data object

	loadct, 0
	tvscl, *img
	obj_destroy, dataSet


	; -------------------------------------------------
	;   Example how to load a multislice time sequence
	; -------------------------------------------------

		; Multislice time sequences occur e.g. in perfusion imaging
		; where a set of n slices is taken every t seconds.

	dataSet = obj_new('mrImageSetDicom')
	file    = dialog_pickfile( path=path, get_path = path )
 	err     = dataSet->loadTimeSequence( file, /progressbar, /rotate )

	if( err eq 0 ) then begin
		x = dialog_message(['Error loading time sequence', data->errMsg()])
		return
	end

	imgs = dataSet->imgData()

		; After loading, mrImageSetDicom orders the time sequence image pointers
		; in a 2-dimensional matrix.
		; The first matrix index denotes the shot number (i.e. the point
		; in time), the second matrix index the slice number
		; imgs thus now is a pointer to an array of image pointers

		; To display the 5th slice in the 3rd shot:

	loadct, 0
	tvscl, *(*imgs)[2,4]

		; When loading very large datasets of a couple of 100 images,
		; destroying the images might be slow!
		; In these cases, it is favourable to call the unload method
		; by hand with the progressbar option set - the destructor
		; calls unload without progressbar option

	dataSet->delete, /all, /progressbar
	obj_destroy, dataSet

		; So much for mrImageSetDicom.
		; The underlying class for data handling is mrDicom, a subclass to
		; to IDLffDicom and covered in the next example.
end
