;+=----------------------------------------------------------------------------
; Name   : mrGetImageSet
; Syntax : sets = mrGetImageSet( imageSets, class=string, name=string, count=count )
; Args   : <b>imageSets</b>
;          An IDL_Container object which holds objects of class mrImageSet or
;          mrImageSetDicom
;          <b>class</b>
;          The class name to filter
;          <b>name</b>
;          The name to filter
;          <b>count</b>
;          If set, returns the number of matching imageSet objects in this variable
; Ret    : obj_new(), if count==0, a single object if count==1 or an array of object
;          references, matching the selection criteria
; Desc   : easier access to a certain image set or class of image sets, given a
;          container in which the sets are stored.
;=-----------------------------------------------------------------------------
function mrGetImageSet, sets, class=class, name=name, count=count
	on_error, 2
	if( n_elements(sets) ne 1 ) then message, 'Missing or wrong argument: sets'
	if( obj_valid(sets)  eq 0 ) then message, 'Invalid IDL_Containter object'
	if( obj_isa( sets, 'IDL_CONTAINER' ) eq 0 ) then message, 'First argument must be an IDL_Container object'

	cl = n_elements(class) eq 0 ? '' : class
	na = n_elements(name)  eq 0 ? '' : name

	matches = n_elements(class) + n_elements(name)
	count   = 0

	all = sets->get( /all, count=nsets )
	if( nsets eq 0 ) then return, obj_new()
	ret = objarr( nsets )
	for i = 0, nsets-1 do begin
		m = 0        ; match
		if( cl ne '' ) then if ( all[i]->class() eq cl ) then m=m+1
		if( na ne '' ) then if ( all[i]->name()	 eq na ) then m=m+1
		if( m eq matches ) then begin
			ret[count] = all[i]
			count=count+1
		end
	end
	case count of
		0: return, obj_new()
		1: return, ret[0]                ; a single object
		else: return, ret[0:count-1]     ; an object array
	end
end
