;+=----------------------------------------------------------------------------
; Name   : mri18n
; RCS    : $Id: mri18n__define.pro,v 2.0 2004/09/06 07:01:26 olivert Exp olivert $
; Methods: <b>General</b>
;          mri18n                          : returns the only instance of this object
;          mri18n->setLocale               : sets the current locale
;          mri18n->get                     : returns a translation - see mrHash::get
;          <br>
;          <b>Private</b>
;          mri18n::init                    : this is a singleton, so init is 'private'
;          mrI18N__define                  : constructor, part I
;          mrI18n::cleanup                 : destructor
; Desc   : I needed some translations in some of the lib's classes. This is
;          a very simple implementation to get something running quickly.
;          Essentially a hash whose values are set programmatically.
;          And a singleton - as far as this is possible in IDL.
; Author: (c) Oliver Thilmann, oliver.thilmann@radfys.lu.se
; Licns : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------

function mri18n
	common mri18n, meMyselfAndI
	if( n_elements(meMyselfAndI) eq 0 ) then begin
		meMyselfAndI = obj_new('mrI18n')
		meMyselfAndI->setLocale, 'en-us'   ; the default
	end
	return, meMyselfAndI
end


;+=----------------------------------------------------------------------------
; Name  : mrI18n::init
; Desc  : Instanciate one object
;=-----------------------------------------------------------------------------
function mrI18n::init
	common mri18n, jaja
	if( n_elements(jaja) ne 0 ) then return, 0  ; singleton
	return, self->mrHash::init( 'string', blocksize=100 )
end


;+=----------------------------------------------------------------------------
; Name : mrI18n::cleanup
; Type : Private Procedure
; Desc : the destructor
;=-----------------------------------------------------------------------------
pro mrI18n::cleanup
	self->mrHash::cleanup
end


;+=----------------------------------------------------------------------------
; Name  : mrI18n::setLocale
; Syntax: obj->setLocale, locale
; Ret   :
;=-----------------------------------------------------------------------------
pro mrI18n::setLocale, locale
	case( strlowcase(locale) ) of
	'de-de': begin
		self->set, 'pb_progress',      'Fortschritt'
		self->set, 'pb_cancel'  ,      'Abbruch'
		self->set, 'pb_complete',      'abgeschlossen.'
		self->set, 'pb_timeRemaining', 'Verbleibende Zeit: '
		self->set, 'pb_seconds',       'Sekunden'

		self->set, 'is_deleting',                 'Lsche Bilder...'
		self->set, 'isd_missingFilename',         'Kein Dateiname angegeben.'
		self->set, 'isd_errOpenFile',             'Fehler beim ffnen der Datei.'
		self->set, 'isd_noSeriesNumber',          'Keine Seriennummer gefunden in '
		self->set, 'isd_missingImgRows',          'Zahl der Bildzeilen nicht gefunden.'
		self->set, 'isd_missingImgCols',          'Anzahl Bildspalten nicht gefunden.'
		self->set, 'isd_missingImgData',          'Fehlende Bilddaten in der Datei '
		self->set, 'isd_loadingImages',           'Lade Bilder...'
		self->set, 'isd_noValidImgsFound',        'Keine Dateien gefunden. Ist der Filter falsch gesetzt?'
		self->set, 'isd_onlyOneImgFound',         'Nur eine gltige Datei gefunden!'
		self->set, 'isd_missingImgNumber',        'In wenigstens einer Datei fehlt die Bildnummer.'
		self->set, 'isd_imgNumbersNotConsecutive','Bildnummern nicht aufeinanderfolgend. '
		self->set, 'isd_firstId',                 'Erste ID:'
		self->set, 'isd_firstMissingId',          'Erste fehlende ID:'
		self->set, 'isd_missingImgDataInImg',     'Keine Bilddaten im Bild '
		self->set, 'isd_scanningImgs',            'Ordne Bilder...'
		self->set, 'isd_missingSlicePosInfo',     'Fehlende Schichtpositionsangabe im Bild '
		self->set, 'isd_imgsBySlicesNotNull',     '#Bilder modulo #Schichten != 0'
		self->set, 'isd_deletingDicomObjs',       'Lsche Dicom-Objekte...'

	end
	'sv-se': begin
		self->set, 'pb_progress',      'Framsteg'
		self->set, 'pb_cancel'  ,      'Avbryt'
		self->set, 'pb_complete',      'komplett.'
		self->set, 'pb_timeRemaining', 'terstende tid: '
		self->set, 'pb_seconds',       'Sekunder'

		self->set, 'is_deleting',                 'Deleting images...'
		self->set, 'isd_missingFilename',         'Missing filename argument.'
		self->set, 'isd_errOpenFile',             'Error opening file'
		self->set, 'isd_noSeriesNumber',          'Could not find series number in file '
		self->set, 'isd_missingImgRows',          'Could not read number of image rows'
		self->set, 'isd_missingImgCols',          'Could not read number of image columns'
		self->set, 'isd_missingImgData',          'Missing image data in file '
		self->set, 'isd_loadingImages',           'Loading images...'
		self->set, 'isd_noValidImgsFound',        'No valid images found - filter setting wrong?'
		self->set, 'isd_onlyOneImgFound',         'Only one valid image found'
		self->set, 'isd_missingImgNumber',        'Failed to retrieve image number of at least one image.'
		self->set, 'isd_imgNumbersNotConsecutive','Image numbers not consecutive. '
		self->set, 'isd_firstId',                 'First id:'
		self->set, 'isd_firstMissingId',          'First missing id:'
		self->set, 'isd_missingImgDataInImg',     'Missing image data in image '
		self->set, 'isd_scanningImgs',            'Scanning images...'
		self->set, 'isd_missingSlicePosInfo',     'Missing slice position info in image'
		self->set, 'isd_imgsBySlicesNotNull',     '#images mod #slices != 0'
		self->set, 'isd_deletingDicomObjs',       'Deleting dicom objects...'
	end
	'fr-fr': begin
		self->set, 'pb_progress',      'Progress'
		self->set, 'pb_cancel'  ,      'Cancel'
		self->set, 'pb_complete',      'Complete.'
		self->set, 'pb_timeRemaining', 'Time remaining: '
		self->set, 'pb_seconds',       'Sekunden'

		self->set, 'is_deleting',                 'Deleting images...'
		self->set, 'isd_missingFilename',         'Missing filename argument.'
		self->set, 'isd_errOpenFile',             'Error opening file'
		self->set, 'isd_noSeriesNumber',          'Could not find series number in file '
		self->set, 'isd_missingImgRows',          'Could not read number of image rows'
		self->set, 'isd_missingImgCols',          'Could not read number of image columns'
		self->set, 'isd_missingImgData',          'Missing image data in file '
		self->set, 'isd_loadingImages',           'Loading images...'
		self->set, 'isd_noValidImgsFound',        'No valid images found - filter setting wrong?'
		self->set, 'isd_onlyOneImgFound',         'Only one valid image found'
		self->set, 'isd_missingImgNumber',        'Failed to retrieve image number of at least one image.'
		self->set, 'isd_imgNumbersNotConsecutive','Image numbers not consecutive. '
		self->set, 'isd_firstId',                 'First id:'
		self->set, 'isd_firstMissingId',          'First missing id:'
		self->set, 'isd_missingImgDataInImg',     'Missing image data in image '
		self->set, 'isd_scanningImgs',            'Scanning images...'
		self->set, 'isd_missingSlicePosInfo',     'Missing slice position info in image'
		self->set, 'isd_imgsBySlicesNotNull',     '#images mod #slices != 0'
		self->set, 'isd_deletingDicomObjs',       'Deleting dicom objects...'
	end
	else: begin

			; mrProgressbar

		self->set, 'pb_progress',      'Progress'
		self->set, 'pb_cancel'  ,      'Cancel'
		self->set, 'pb_complete',      'Complete.'
		self->set, 'pb_timeRemaining', 'Time remaining: '
		self->set, 'pb_seconds',       'Seconds'

			; mrImageSet and mrImageSetDicom

		self->set, 'is_deleting',                 'Deleting images...'
		self->set, 'isd_missingFilename',         'Missing filename argument.'
		self->set, 'isd_errOpenFile',             'Error opening file'
		self->set, 'isd_noSeriesNumber',          'Could not find series number in file '
		self->set, 'isd_missingImgRows',          'Could not read number of image rows'
		self->set, 'isd_missingImgCols',          'Could not read number of image columns'
		self->set, 'isd_missingImgData',          'Missing image data in file '
		self->set, 'isd_loadingImages',           'Loading images...'
		self->set, 'isd_noValidImgsFound',        'No valid images found - filter setting wrong?'
		self->set, 'isd_onlyOneImgFound',         'Only one valid image found'
		self->set, 'isd_missingImgNumber',        'Failed to retrieve image number of at least one image.'
		self->set, 'isd_imgNumbersNotConsecutive','Image numbers not consecutive. '
		self->set, 'isd_firstId',                 'First id:'
		self->set, 'isd_firstMissingId',          'First missing id:'
		self->set, 'isd_missingImgDataInImg',     'Missing image data in image '
		self->set, 'isd_scanningImgs',            'Scanning images...'
		self->set, 'isd_missingSlicePosInfo',     'Missing slice position info in image'
		self->set, 'isd_imgsBySlicesNotNull',     '#images mod #slices != 0'
		self->set, 'isd_deletingDicomObjs',       'Deleting dicom objects...'
	end
	end
end


;+=----------------------------------------------------------------------------
; Name : mrI18n__define
; Type : Private procedure
; Desc : The constructor
;=-----------------------------------------------------------------------------
pro mri18n__define
	obj = { MRI18N       		  ,$
			inherits MRHASH        $
		}
end
