;+=----------------------------------------------------------------------------
; Name    : mod_dicomtools
; RCS     : $Id: mod_dicomtools__define.pro,v 1.2 2004/11/25 09:06:39 olivert Exp olivert $
; Desc    : A tool for handling of DICOM files, independent of a special application.
;           Both, UI based and non-UI based methods may be located here.
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_dicomtools::revision
	rev = '$Revision: 1.2 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::init
; Desc  :
;=-----------------------------------------------------------------------------
function mod_dicomtools::init, g

    res = self->lupemod::init( 'mod_dicomtools', 'top', 0, g )
    if( res eq 0 ) then return, 0

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::readyToExecute
; Desc  : we're always ready to execute
;=-----------------------------------------------------------------------------
function mod_dicomtools::readyToExecute, mode
	i18n = (*self.g).i18n
	if( self.mode ne '' ) then begin
		x = dialog_message( i18n->get('err_instanceAlreadyRunning') )
		return, 0
	end
	if( mode ne 'resortDicomFilesBySeriesnumber' and mode ne 'splitMosaicFiles' and $
	    mode ne 'anonymizeDirectory' ) then begin
		x = dialog_message( 'Unknown mode ' + mode + ' in mod_dicomtools.', /error )
		return, 0
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::map
; Desc  : As of now, there's only one mode...
;=-----------------------------------------------------------------------------
pro mod_dicomtools::map, mode

	g     = self.g              ; pointer to global structure
	prefs = (*g).preferences    ; pointer to preferences structure
	i18n  = (*g).i18n

	case mode of
	    'resortDicomFilesBySeriesnumber':  begin
	    	res = self->createBaseWidget( /column, title=i18n->get('dcm_resortBySeriesNumber'), scr_xsize=484, scr_ysize=186 )
	    	buttonTxt = i18n->get('dcm_sort')
	    end
	    'anonymizeDirectory': begin
			res = self->createBaseWidget( /column, title=i18n->get('dcm_anonymizeFiles'), scr_xsize=484, scr_ysize=186 )
			buttonTxt = i18n->get('dcm_anonymize')
		end
	    'splitMosaicFiles': begin
	    	res = self->createBaseWidget( /column, title=i18n->get('dcm_splitMosaicFiles'), scr_xsize=484, scr_ysize=186 )
	    	buttonTxt = i18n->get('dcm_split')
	    end
	end

	f1  = widget_base( self.baseid, xpad=10, ypad=10, /column, /frame )

	x   = widget_label( f1, value=i18n->get('dcm_selectDirectories'), /align_left )
	f2  = widget_base( f1, /column, /frame )
	r1  = widget_base( f2, /row )
	x   = widget_label ( r1, value=i18n->get('dcm_sourceDirectory'), scr_xsize=100 )
	x   = widget_text  ( r1, value='', scr_xsize=300 )
	self.idSourceDir = x
	x   = widget_button( r1, value='...', uvalue={o:self, m:'sourceDirectorySelectEvt'} )
	r1  = widget_base( f2, /row )
	x   = widget_label ( r1, value=i18n->get('dcm_targetDirectory'), scr_xsize=100 )
	x   = widget_text  ( r1, value='', scr_xsize=300 )
	self.idTargetDir = x
	x   = widget_button( r1, value='...', uvalue={o:self, m:'targetDirectorySelectEvt'} )

	x  = widget_label( f1, value='', scr_ysize=10 )
	r1  = widget_base( f1, /row )

	x   = widget_label ( r1, value='', scr_xsize=80 )
	x   = widget_button( r1, scr_xsize=80, value=i18n->get('Cancel'), uvalue={o:self, m:'cancelButtonEvt'} )
	x   = widget_label ( r1, scr_xsize=120, value='' )
	x   = widget_button( r1, scr_xsize=80, value=buttonTxt , uvalue={o:self, m:'sortButtonEvt'}, /frame )

	self.mode = mode
	self->placeAndRegister

	return
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::unmap
; Desc  : call of this function is very likely a result of the user pressing
;         the close handle - i.e. we'll do some cleanup here
;=-----------------------------------------------------------------------------
pro mod_dicomtools::unmap, _ref_extra=ex
	self.mode = ''
	if( obj_valid( self.pb ) ) then begin
		obj_destroy, self.pb
		self.pb = obj_new()
	end
	self->lupemod::unmap, _strict_extra=ex
end

;==============================================================================
;
;                                  EVENT HANDLING
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::sourceDirectorySelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_dicomtools::sourceDirectorySelectEvt, evt
	i18n = (*self.g).i18n
	dir = dialog_pickfile( title=i18n->get('dcm_selectSourceDirectory'), /must_exist, /directory, path=(*(*self.g).preferences).workingDir )
	if( dir eq '' ) then return
	widget_control, self.idSourceDir, set_value=dirname(dir)  ; IDL's file_dirname is crap - in fact it's so braindead, it almost makes me angry
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::targetDirectorySelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_dicomtools::targetDirectorySelectEvt, evt
	i18n = (*self.g).i18n
	dir = dialog_pickfile( title=i18n->get('dcm_selectTargetDirectory'), path=(*(*self.g).preferences).workingDir )
	if( dir eq '' ) then return
	widget_control, self.idTargetDir, set_value=dirname(dir)
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::sortButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_dicomtools::sortButtonEvt, evt
	i18n = (*self.g).i18n
	widget_control, self.idSourceDir, get_value=sourceDir
	widget_control, self.idTargetDir, get_value=targetDir
	if( sourceDir eq '' ) then begin
		x = dialog_message(i18n->get('dcm_pleaseSelectSourceDir'), /info )
		return
	end
	if( targetDir eq '' ) then begin
		x = dialog_message(i18n->get('dcm_pleaseSelectTargetDir'), /info )
		return
	end

	case self.mode of
	    'resortDicomFilesBySeriesnumber':  res = self->resortDicomDir( sourceDir, targetDir )
	    'splitMosaicFiles':                res = self->splitMosaicFiles( sourceDir, targetDir )
	    'anonymizeDirectory':              res = self->anonymizeDirectory( sourceDir, targetDir )
	end

	if( res eq 1 ) then self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::cancelButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_dicomtools::cancelButtonEvt, evt
	self->unmap
end


;------------------------------------------------------------------------------
;
;                         F U N C T I O N A L I T Y
;
;------------------------------------------------------------------------------

function mod_dicomtools::getTagValue, dicom, group, element
	vals    = dicom->getValue( group, element, /no_copy )
	refs    = dicom->getReference( group, element )
	parents = dicom->getParent( refs )
	id = where( parents eq -1 )
	return, vals[id[0]]
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::resortDicomDir
; Desc  :
;=-----------------------------------------------------------------------------
function mod_dicomtools::resortDicomDir, sourceDir, targetDir

	i18n = (*self.g).i18n

		; error handling

	error = 0
	catch, error
	if( error ne 0 ) then begin
		catch, /cancel
		if( obj_valid(dicom) ) then obj_destroy, dicom
		if( obj_valid( self.pb ) ) then begin
			obj_destroy, self.pb
			self.pb=obj_new()
		end
		x = dialog_message( !ERROR_STATE.msg, /error )
		return, 0
	end

		; the method

	allfiles = findfile( sourcedir+'\*', count=nfiles )
	if( nfiles eq 0 ) then message, 'No files in source directory' ; can't happen, there's always '.' and '..'

	self.pb = obj_new('mrProgressbar')

	for i = 0, nfiles-1 do begin
		self.pb->update, (i+1.)/nfiles

		dicom = obj_new('idlffdicom', allfiles[i] )
		if( dicom eq obj_new() ) then continue       ; loading failed

		valptr = self->getTagValue( dicom, 32, 17 )  ; series number
		if( not ptr_valid(valptr) ) then begin
			obj_destroy, dicom
			continue
		end
		seriesno = *valptr

		valptr = self->getTagValue( dicom, 8, 4158 ) ; series description
		if( not ptr_valid(valptr) ) then begin
			obj_destroy, dicom
			continue
		end
		seriesdesc = *valptr

			; if .dcm extension is missing, add it to filename
			; if target directory does not exist, create it

		file = strtrim(file_basename(allfiles[i]),2)
		if( strpos( allfiles[i], '.dcm', /reverse_search ) ne strlen(allfiles[i])-4 ) then file = file+'.dcm'

		tdir = targetDir + '\' + strtrim(seriesno,2) + '_' + strtrim(seriesdesc,2)
		tfil = tdir+'\'+file
		if( not file_test( tdir, /directory ) ) then file_mkdir, tdir

		file_copy, allfiles[i], tfil, /noexpand_path, /overwrite
		obj_destroy, dicom

	end
	obj_destroy, self.pb
	self.pb = obj_new()
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::splitMosaicFiles
; Desc  : This does not work (functionality handled in ASL module)
;=-----------------------------------------------------------------------------
function mod_dicomtools::splitMosaicFiles, sourceDir, targetDir

	x = dialog_message('Not properly working...')
	return, 0

	allfiles = findfile( sourceDir+'\*.dcm', count=nfiles )

		; load the sequence

	set = obj_new('mrImagesetDicom')
	res = set->loadTimeSequence( allfiles[0], /progressbar )
	if( res eq 0 ) then begin
		x = dialog_message('Error loading sequence: ' + set->errMsg() )
		obj_destroy, res
		return, 0
	end
	nShots = set->nShots()

		; get the number of images in the mosaic

	img = *(*(set->imgData()))[0,0]
	nSlices = 0
	for y = 0, 2 do begin
		for x = 0, 2 do begin
			if( total( img[x*64:x*64+63,y*64:y*64+63] ) gt 0. ) then nSlices++
		end
	end

		; create dicom image sets

	sets = objarr(2)
	for i = 0, 1 do begin
		sets[i] = obj_new('mrimageset', nSlices, nShots/2, xsize=64, ysize=64 )
	end

		; get sorting order by acquitision time

	imgtime = fltarr( nShots )
	dicom = set->dicomData()
	for i = 0, nShots-1 do begin
		valptr = self->getTagValue( (*dicom)[0,i], 8, 50 ) ; acquisition time
		imgtime[i] = *valptr
	end
	sortid = sort(imgtime)

		; loop over all times

	for shot = 0, nShots-1 do begin
		srcImg = (*(set->imgData()))[0,sortid[shot]]
		for slice = 0, nSlices-1 do begin
			x = (slice mod 3 ) * 64
			y = (slice / 3)*64
			;img = intarr(64,64)
			img = (*srcImg)[x:x+63,y:y+63]
			sets[shot mod 2]->set, [slice, shot/2], ptr_new(img, /no_copy), /no_copy
		end
	end

	obj_destroy, sets[0]
	obj_destroy, sets[1]

	return, 0
end

;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools::anonymizeDirectory
; Desc  :
;=-----------------------------------------------------------------------------
function mod_dicomtools::anonymizeDirectory, sourceDir, targetDir

	i18n = (*self.g).i18n

		; error handling

	error = 0
	catch, error
	if( error ne 0 ) then begin
		catch, /cancel
		if( obj_valid(dicom) ) then obj_destroy, dicom
		if( obj_valid( self.pb ) ) then begin
			obj_destroy, self.pb
			self.pb=obj_new()
		end
		x = dialog_message( !ERROR_STATE.msg, /error )
		return, 0
	end

		; anonymize

	allfiles = findfile( sourcedir+'\*', count=nfiles )
	if( nfiles eq 0 ) then message, 'No files in source directory'

	self.pb = obj_new('mrProgressbar')

	for i = 0, nfiles-1 do begin
		self.pb->update, (i+1.)/nfiles

		dicom = obj_new('mrdicom', allfiles[i] )
		if( dicom eq obj_new() ) then continue       ; loading failed

		dicom->anonymize
		file = targetDir + '\' + strtrim( file_basename(allfiles[i]), 2)
		res = dicom->write( file )
		if( res eq 0 ) then print, 'Writing of file ' + file + ' failed!'
		obj_destroy, dicom

	end
	obj_destroy, self.pb
	self.pb = obj_new()
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_dicomtools__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_dicomtools__define
	obj = { mod_dicomtools     				,$
			inherits LUPEMOD      			,$
			idSourceDir:			   0L   ,$ ; widget id of source dir text box
			idTargetDir:			   0L   ,$ ; widget id of target dir text box
			mode:                      ''   ,$ ; current mode
			pb:					obj_new()	 $ ; a progressbar widget
		  }
end

