;+=----------------------------------------------------------------------------
; Name    : mod_loadresultset
; RCS     : $Id: mod_loadresultset__define.pro,v 1.1 2004/11/11 11:33:47 olivert Exp olivert $
; Desc    : Loads a result set
;           For now we use a very simple workflow and UI layout
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_loadresultset::revision
	rev = '$Revision: 1.1 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::init
; Desc  :
;=-----------------------------------------------------------------------------
function mod_loadresultset::init, g

    res = self->lupemod::init( 'mod_loadresultset', 'top', 0, g )
    if( res eq 0 ) then return, 0

	self.newset = obj_new()

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::readyToExecute
; Desc  : We're always ready to execute
;=-----------------------------------------------------------------------------
function mod_loadresultset::readyToExecute, option
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::map
; Desc  : Workflow:
;         Present a fileselector box to the user
;         Load the sequence
;         Check, whether this is a sequence as we expect
;         Check whether the studyUID matches existing data (if necessary)
;         If we got that far, finally map the UI which:
;         - asks the user for a name for this image set - suggest the
;           filename (without _s??) as default
;         - if the name already exists, warn the user and ask whether to
;           replace the existing set or chose a different name
;=-----------------------------------------------------------------------------
pro mod_loadresultset::map, option

	g     = self.g              ; pointer to global structure
	prefs = (*g).preferences    ; pointer to preferences structure
	i18n  = (*g).i18n

		; select a file, instanciate the object and load the images

	file = dialog_pickfile( path=(*g).workingdir, get_path=path, /must_exist, /read, title=i18n->get('lrs_selectFile') )
	if( file eq '' ) then return

	newset   = obj_new('mrimagesetdicom', class='result' )
	res      = newset->loadSequence( file, /progressbar, /rotate )
	if( res eq 0 ) then begin
		errtxt = newset->errMsg()
		res = dialog_message([i18n->get('err_loadingSequence')+':', errtxt], title=i18n->get('Error'), /error)
		obj_destroy, newset
		return
	end
	(*g).workingdir=path

		; check whether this image set looks like a result image set

	if( newset->nSlices() eq 1 ) then begin
		res = mrdialog_select([i18n->get('lrs_onlyOneImageLoaded1'), i18n->get('lrs_onlyOneImageLoaded2')], $
		                      [i18n->get('lrs_onlyOneLoadedYes'), i18n->get('lrs_onlyOneLoadedNo')], $
		                       group_leader=(*g).topid, /warning, title=i18n->get('lrs_suspiciousDataLayout') )
		if( res eq 1 ) then begin
			obj_destroy, newset
			return
		end
	end

	if( newset->nSlices() gt 100 ) then begin
		res = mrdialog_select([i18n->get('lrs_tooManySlices1'), i18n->get('lrs_tooManySlices2'), i18n->get('lrs_tooManySlices3')], $
		                      [i18n->get('lrs_tooManySlicesYes'), i18n->get('lrs_tooManySlicesNo')],$
		                      group_leader=(*g).topid, /warning, title=i18n->get('lrs_suspiciousDataLayout') )
		if( res eq 1 ) then begin
			newset->delete, /all, /progressbar
			obj_destroy, newset
			return
		end
	end

		; check whether the study UID of the loaded data matches the current studyUID

	if( (*prefs).checkstudyUID eq 1 ) then begin
		allsets = (*g).imgSets->get( /all, count=nsets )
		for i = 0, nsets-1 do begin
			if( obj_isa( allsets[i], 'mrImageSetDicom' )) then begin
				if( newset->studyUID() ne allsets[i]->studyUID() ) then begin
					res = mrdialog_select([i18n->get('lrs_studyUIDdoesNotMatch1'), i18n->get('lrs_studyUIDdoesNotMatch2'),$
										   i18n->get('lrs_studyUIDdoesNotMatch3')],$
				 	                       'Ok', /error, group_leader=(*g).topid, title=i18n->get('Error') )
					newset->delete, /all, /progressbar
			   		obj_destroy, newset
			   		return
			   	end
			end
		end
	end

	dcm = (*(newset->dicomData()))[0]
	self.newset = newset

			; Now finally map the user interface to ask the user for a name for this set

	res = self->createBaseWidget( /column, title=i18n->get('lrs_selectSetName'), scr_xsize=360, scr_ysize=288 )
	if( res eq 0 ) then message, 'Error creating base widget' ; very unlikely

	r1 = widget_base ( self.baseid, xpad=10, ypad=10, /column, /frame )
	x  = widget_label( r1, value=i18n->get('lrs_imageSetInformation'), /align_left )
	r2 = widget_base ( r1, /frame, /column )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, value=i18n->get('lrs_numberOfImages'), scr_xsize=150 )
	x  = widget_label( r3, scr_xsize=150, value=strtrim(newset->d0(),2))
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, value=i18n->get('lrs_fileBasename'), scr_xsize=150 )
	x  = widget_label( r3, scr_xsize=150, value=file_basename(dcm->filename()) )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, value=i18n->get('lrs_seriesDescription'), scr_xsize=150 )
	x  = widget_label( r3, scr_xsize=150, value=dcm->getTagValue('id_series_description'))

	x  = widget_label( r1, value='', scr_ysize=10 )

	x  = widget_label( r1, value=i18n->get('lrs_imageSetName'), /align_left )
	r2 = widget_base ( r1, /frame, /column )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, value=i18n->get('lrs_selectNameForSet'), scr_xsize=150 )
	self.idtext = widget_text ( r3, scr_xsize=150, /editable, /all_events, uvalue={o:self, m:'keyEvt'} )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, value=i18n->get('lrs_suggestBasedOn'), scr_xsize=150 )
	x  = widget_button( r3, value=i18n->get('lrs_filename'), scr_xsize=75, uvalue={o:self, m:'suggestFilenameEvt'} )
	x  = widget_button( r3, value=i18n->get('lrs_description'), scr_xsize=75, uvalue={o:self, m:'suggestDescriptionEvt'} )

	x  = widget_label( r1, value='', scr_ysize=10 )

	r2 = widget_base(  r1, /row )
	x  = widget_label ( r2, value='', scr_xsize=30 )
	x  = widget_button( r2, value=i18n->get('Cancel'), scr_xsize=90, uvalue={o:self, m:'cancelButtonEvt'} )
	x  = widget_label ( r2, value='', scr_xsize=60 )
	x  = widget_button( r2, value='Ok', scr_xsize=90, /frame, uvalue={o:self, m:'okButtonEvt'} )

	self->placeAndRegister
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::unmap
; Desc  : call of this function is very likely a result of the user pressing
;         the close handle - i.e. we'll do some cleanup here
;=-----------------------------------------------------------------------------
pro mod_loadresultset::unmap, _ref_extra=ex
	if( obj_valid(self.newset) ) then begin
		self.newset->delete, /all, /progressbar
		obj_destroy, self.newset
		self.newset = obj_new()
	end
	self->lupemod::unmap, _strict_extra=ex
end

;==============================================================================
;
;                                  EVENT HANDLING
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::keyEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_loadresultset::keyEvt, evt
	if( evt.type eq 0 ) then begin
		if( evt.ch eq 10 ) then begin
			if( self->addSet() eq 1 ) then self->unmap
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::suggestFilenameEvt
; Desc  : In the two suggest functions, we essentially try to reverse the
;         default settings as made in mod_save.
;         For the filename, we replace the underscores by blanks and
;         remove a trailing _s??.dcm
;=-----------------------------------------------------------------------------
pro mod_loadresultset::suggestFilenameEvt, evt
	name = file_basename((*(self.newset->dicomData()))[0]->filename())
	ext = strpos( name, '.', /reverse_search )            ; cut off extension
	if( ext ne -1 ) then name = strmid( name, 0, ext )
	parts = strsplit( name, '_', /extract )
	nparts = n_elements(parts)
	if( nparts gt 1 and strpos( parts[nparts-1],'s') eq 0 ) then nparts=nparts-1
	name  = strjoin( parts[0:nparts-1], ' ' )
	widget_control, self.idtext, set_value=name
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::suggestDescriptionEvt
; Desc  : As above.
;         Here we remove a leading 'Calculated ' and a trailing version
;         information.
;=-----------------------------------------------------------------------------
pro mod_loadresultset::suggestDescriptionEvt, evt
	dcm = (*(self.newset->dicomData()))[0]
	desc = dcm->getTagValue('id_series_description')
		; remove at least the stuff we added in mod_save, i.e. a
		; leading 'Calculated ' and a trailing 'Lupe V...'
	ext = strpos( desc, 'Calculated ' )
	if( ext eq 0 ) then desc = strmid( desc, 11, strlen(desc) )
	ext = strpos( desc, 'Lupe V' )
	if( ext ne -1 ) then desc = strmid( desc, 0, ext )
	widget_control, self.idtext, set_value=desc
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::cancelButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_loadresultset::cancelButtonEvt, evt
	self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::okButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_loadresultset::okButtonEvt, evt
	if( self->addSet() eq 1 ) then self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset::addSet
; Desc  :
;=-----------------------------------------------------------------------------
function mod_loadresultset::addSet
	i18n = (*self.g).i18n
	widget_control, self.idtext, get_value=name
	if( name eq '' ) then begin
		x = dialog_message(i18n->get('lrs_pleaseSelectNameWarn'), /info, title=i18n->get('Info'))
		return, 0
	end
	x = mrGetImageSet( (*self.g).imgSets, class='result', name=name, count=count )
	if( count ne 0 ) then begin
		x = dialog_message([i18n->get('lrs_setnameAlreadyExists1'), i18n->get('lrs_setnameAlreadyExists2')], /info, title=i18n->get('Info'))
		return, 0
	end
	self.newset->setName, name
	(*self.g).imgSets->add, self.newset
	self.newset = obj_new()
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_loadresultset__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_loadresultset__define
	obj = { MOD_loadresultset     	,$
			inherits LUPEMOD      	,$
			idtext:			   0L   ,$ ; widget id of the text box
			newset:		obj_new() 	 $ ; the loaded set
		  }
end

