;+=----------------------------------------------------------------------------
; Name    : mod_rsvd
; RCS     : $Id: mod_rsvd__define.pro,v 1.3 2004/11/25 09:08:11 olivert Exp olivert $
; Desc    : This module is 98% copy'n paste from mod_ssvd, except for the
;           additional mrwidget_numfield and the slightly modified calculation
;           routines.
;           The calculation method is described in
;           M.R.Smith, H.Lu, S.Trochet, R.Frayne
;           'Removing the effect of SVD algorithmic artifacts present in
;           quantitative MR perfusion studies', MRM 51:631-634 (2004)
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_rsvd::revision
	rev = '$Revision: 1.3 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_rsvd::init, g

    res = self->lupemod::init( 'mod_rsvd', 'embedded', 0, g )
    if( res eq 0 ) then return, 0

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::readyToExecute
; Desc  : Checks whether AIF is available
;=-----------------------------------------------------------------------------
function mod_rsvd::readyToExecute, option
	i18n = (*self.g).i18n

	sigset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	ctrset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt')
	baselineset = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline')
	if( sigset eq obj_new() or ctrset eq obj_new() or baselineset eq obj_new() ) then begin
		x = dialog_message(i18n->get('svd_missingRawImgSet'), title=i18n->get('Information'))
		return, 0
	end

	aifset = mrGetImageSet( (*self.g).imgsets, class='aif', name='global' )
	if( aifset eq obj_new() ) then begin
		x = dialog_message(i18n->get('svd_noAIFDefined'), title=i18n->get('Info'))
		return, 0
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::map
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_rsvd::map, option

	i18n = (*self.g).i18n

		; create the base widget

	res = self->createBaseWidget( /row )
	if( res eq 0 ) then return

	l = widget_base( self.baseid, /column, /frame )
	r = widget_base( self.baseid, /column, /frame, /base_align_center )

		; left side: multidisplay & three result displays

	imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
    nrows = 6 > ( imgset->nshots() + 9)/10


	l1 = widget_base( l, /row )
	self.mdisp = obj_new('mrmultidisplay', l1, 700, 420, 70, 70, 10, nrows )
	l1 = widget_base( l, /row )
	self.disp1 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )
	self.disp2 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )
	self.disp3 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )

		; right side: buttons & info

    x  = widget_label( r, value=i18n->get('svd_rsvdTitle'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=250)
	r1 = widget_base ( r, xsize=230, /column )

		; image set names

	x  = widget_base ( r1, ysize=20 )
	r2 = widget_base ( r1, /row )
	x  = widget_label( r2, xsize=200, value=i18n->get('svd_resultImageSetNames') )
	r2 = widget_base ( r1, xsize=220, /column, /frame )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_CBVname') )
	self.idcbvname  = widget_text ( r3, xsize=130, value='rCBV rSVD', /editable, uvalue={o:self, m:'dummyEvt'})
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_CBFname') )
	self.idcbfname  = widget_text ( r3, xsize=130, value='rCBF rSVD', /editable, uvalue={o:self, m:'dummyEvt'} )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_MTTname') )
	self.idmttname  = widget_text ( r3, xsize=130, value='rMTT rSVD', /editable, uvalue={o:self, m:'dummyEvt'} )

		; Display parameters

	x  = widget_base ( r1, ysize=20 )
	r2 = widget_base ( r1, /row )
	x  = widget_label( r2, xsize=200, value=i18n->get('svd_currentParameters') )

	r2 = widget_base ( r1, xsize=220, /column, /frame )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_signalValueThres') )
	self.idsigthres  = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_ctrValueThres') )
	self.idctrthres  = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_wMatrixCutoff') )
	self.idwcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_CBVcutoff') )
	self.idcbvcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_CBFcutoff') )
	self.idcbfcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_MTTcutoff') )
	self.idmttcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	self.idtimeshift   = mrwidget_numfield( r2, /integer, value=0, min=0, max=100, incr=1, xsize=200, labelsize=145, fieldsize=40, $
											labeltext=i18n->get('svd_rsvdTimeshift'), /editable, /all_events, uvalue={o:self, m:'dummyEvt'} )

		; progressbar and calculate button

	x  = widget_base  ( r1, ysize=57 )
	x  = widget_button( r1, xsize=230, value=i18n->get('svd_calculate'), uvalue={o:self, m:'calcButtonEvt'})
	self.progressbar = obj_new( 'mrprogressbar', r1, xsize=222, /frame, title=i18n->get('CalculationProgress'), /remaining )

		; close window button

	x  = widget_base  ( r1, ysize=136 )
	x  = widget_button( r1, xsize=220, value=i18n->get('CloseModule'), uvalue={o:self, m:'closeButtonEvt'} )

       ; set labels on the three result images

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'rCBV', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp1->labelView())->add, mdl

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'rCBF', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp2->labelView())->add, mdl

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'MTT', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp3->labelView())->add, mdl

		; realize the widget and fill in the parameters

	self->placeAndRegister
	self->updateParameters

end

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::unmap
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_rsvd::unmap, _ref_extra=ex
	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

	obj_destroy, self.mdisp
	obj_destroy, self.disp1
	obj_destroy, self.disp2
	obj_destroy, self.disp3

	self->lupemod::unmap, _strict_extra=ex
end


;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::calcButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_rsvd::calcButtonEvt, evt
	self->calculate
end

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_rsvd::closeButtonEvt, evt
	self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::dummyEvt
; Desc  : catch events which are not used as widget fields are readout
;         directly when used
;=-----------------------------------------------------------------------------
pro mod_rsvd::dummyEvt, evt
end



;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::updateParameters
; Desc  : Get the parameters from preferences and display them
;=-----------------------------------------------------------------------------
pro mod_rsvd::updateParameters
	prefs = (*self.g).preferences

	widget_control, self.idsigthres,  set_value=string( (*prefs).ssvd_sigthres, format='(f6.1)' )
	widget_control, self.idctrthres,  set_value=string( (*prefs).ssvd_ctrthres, format='(f6.1)' )
	widget_control, self.idwcutoff,   set_value=string( fix( 100.*(*prefs).ssvd_matrixcutoff) )
	widget_control, self.idcbvcutoff, set_value=string( (*prefs).ssvd_cbvcutoff, format='(f5.0)' )
	widget_control, self.idcbfcutoff, set_value=string( (*prefs).ssvd_cbfcutoff, format='(f5.0)' )
	widget_control, self.idmttcutoff, set_value=string( (*prefs).ssvd_mttcutoff, format='(f5.0)' )
end

;+=----------------------------------------------------------------------------
; Name  : mod_rsvd::calculate
; Desc  : Perform the SVD calculation, slice by slice, and display the results
;         after each slice
;=-----------------------------------------------------------------------------
pro mod_rsvd::calculate

	i18n = (*self.g).i18n

	self->updateParameters
	self.progressbar->reset

	sigset      = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	ctrset      = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
	baselineset = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline')

	if( sigset eq obj_new() or ctrset eq obj_new() or baselineset eq obj_new() ) then begin
		x = dialog_message([i18n->get('svd_missingSets1'),i18n->get('svd_missingSets2')], /error, title=i18n->get('Error') )
		self->unmap
		return
	end

	nslices = sigset->nslices()
	xsize   = sigset->xsize()
	ysize   = sigset->ysize()

		; introduce a 2 bin delay in half of the ctr data for testing

;	imgs = *(ctrset->imgdata())
;	nslices = ctrset->d0()
;	for i = 0, nslices-1 do begin
;		(*imgs[i])[0:63,*,2:59] = (*imgs[i])[0:63,*,0:57]
;	end
;
;	return

		; check whether the chosen name is already present in memory

	resultsets = mrGetImageSet( (*self.g).imgSets, class='result', count=nsets )
	widget_control, self.idcbvname, get_value=cbvname
	cbvname = strtrim(cbvname, 2)
	widget_control, self.idcbfname, get_value=cbfname
	cbfname = strtrim(cbfname, 2)
	widget_control, self.idmttname, get_value=mttname
	mttname = strtrim(mttname, 2)
	if( cbvname eq '' or cbfname eq '' or mttname eq '' ) then begin
		x = dialog_message(i18n->get('svd_missingSetName'), title=i18n->get('Error'), /error)
		return
	end
	names = [cbvname, cbfname, mttname]
	if( n_elements( uniq(names[sort(names)]) ) ne 3 ) then begin
		x = dialog_message(i18n->get('svd_imageSetNamesMustDiffer'), title=i18n->get('Error'), /error)
		return
	end
	if( nsets gt 0 and (*(*self.g).preferences).warnResultReplace eq 1 ) then begin
		setnames = resultSets[0]->name()
		for i = 1, nsets-1 do setnames = [setnames, resultSets[i]->name()]
		doubles = intersect( names, setnames )
		if( doubles[0] ne -1 ) then begin
			x = mrdialog_select( [i18n->get('svd_resultSetExistsWarn1'),doubles,i18n->get('svd_resultSetExistsWarn2'),$
								  i18n->get('svd_resultSetExistsWarn3')], title=i18n->get('Question'), $
								 ['Yes', 'No'], default=1, group_leader=self.baseid, bsize=80, /question )
			if( x eq 1 ) then return
		end
	end

		; create the result sets

	cbvset = mrGetImageSet( (*self.g).imgSets, class='result', name=cbvname )
	cbfset = mrGetImageSet( (*self.g).imgSets, class='result', name=cbfname )
	mttset = mrGetImageSet( (*self.g).imgSets, class='result', name=mttname )

	if( obj_valid(cbvset) ) then begin
		(*self.g).imgSets->remove, cbvset
		obj_destroy, cbvset
	end
	if( obj_valid(cbfset) ) then begin
		(*self.g).imgSets->remove, cbfset
		obj_destroy, cbfset
	end
	if( obj_valid(mttset) ) then begin
		(*self.g).imgSets->remove, mttset
		obj_destroy, mttset
	end

	cbvset = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=cbvname )
	cbfset = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=cbfname )
	mttset = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=mttname )

		; get pointer to AIF

	aifset = mrGetImageSet( (*self.g).imgsets, class='aif', name='global' )
	if( aifset eq obj_new() ) then begin       ; just for safety, already checked in 'readyToExecute'
		x = dialog_message(i18n->get('svd_noAIFDefined'), title=i18n->get('Error'), /error)
		return
	end
	aif = (*(aifset->imgData()))[0]

		; get parameters from data - if the DICOM data contains no TR, we use the default value
		; without warning. The user has already been warned about that when calculating concentration
		; images

	dicom = (*(sigset->dicomData()))[0,0]
	tr    = dicom->getTagValue('acq_repetition_time')
	if( tr eq '' ) then tr = (*(*self.g).preferences).default_tr  ; take from defaults, if not found
	tr    = tr/1000.                                              ; convert time into seconds

		; perform the calculation

	imgdata = sigset->imgdata()

	widget_control, self.idtimeshift, get_value=timeshift

	for i = 0, nslices-1 do begin

		self.mdisp->label, /delete, /all
		for k = 0, sigset->nshots()-1 do begin
			row = k/10
			col = k mod 10
			self.mdisp->setImg, col, row, (*imgdata)[i, k], /no_contrastupdate
			self.mdisp->label, col, row, string( k, format='(i2)' )
		end
		self.mdisp->setContrast  ; to update scaling
		self.mdisp->display

		self->calcPerfusionImages, i, tr, aif, timeshift, baselineset, sigset, ctrset, cbvset, cbfset, mttset
		self.progressbar->update, (i+1.)/nslices
		self.disp1->setImg, (*(cbvset->imgdata()))[i]  ; loops? what loops?
		self.disp1->fitToScreen
		self.disp1->display
		self.disp2->setImg, (*(cbfset->imgdata()))[i]
		self.disp2->fitToScreen
		self.disp2->display
		self.disp3->setImg, (*(mttset->imgdata()))[i]
		self.disp3->fitToScreen
		self.disp3->display
	end

		; Honor thy father and thy mother: that thy days may be long

	cbvset->setParent, sigset
	cbfset->setParent, sigset
	mttset->setParent, sigset

		; add the image sets to the central container

	(*self.g).imgSets->add, cbfset
	(*self.g).imgSets->add, cbvset
	(*self.g).imgSets->add, mttset

end

;+=----------------------------------------------------------------------------
; Name   : mod_rsvd::calcPerfusionImages
; Desc   : Calculates the perfusion images via reformulated SVD
;          as described by Smith et al., MRM 51 (2004), 631-634
;=-----------------------------------------------------------------------------
pro mod_rsvd::calcPerfusionImages, slice, tr, ai, dt, baselineset, sigset, ctrset, cbvset, cbfset, mttset

	aif = fltarr( n_elements(*ai) + dt )
	aif[0:n_elements(*ai)-1] = *ai

	nshots       = sigset->nshots()
	nshotsdt     = nshots+fix(dt)
	xsize        = sigset->xsize()
	ysize        = sigset->ysize()
	matrixcutoff = (*(*self.g).preferences).ssvd_matrixcutoff
	sigthres     = (*(*self.g).preferences).ssvd_sigthres
	bl           = baselineset->uvalue()  ; region over which average was calculated

       ; preparation of the arterial input function - make local copy and
       ; have the aif always be positive.
       ; As indices up to ns0[1] are used for baseline averaging, the
       ; corresponding values are not used - thus all field and matrix sizes
       ; within this routine are of length nshots - ns0[1] - 1

    aif  = aif[(*bl)[1]+1:*]
    aif  = aif > 0
    aifc = aif              ; aif with cut off recirculation effects

       ; cut end of aif function to get rid of recirculation effects
       ; achieved by setting all values to zero after bump at a level < 40% maxvalue
       ; according to Ronnie, this doesn't make much of a difference, though

    maxval = max( aifc, maxid )
    for i = maxid+1, n_elements(aifc)-1 do begin
       if( aifc[i] le aifc[i-1] or aifc[i] gt .4*maxval ) then continue
       aifc[i:n_elements(aifc)-1] = 0.
       break
    end

       ; create the diagonal matrix for unfolding - see the Oestergaard paper
       ; the dimension of the aifmatrix does not need to include the leading (*bl)[1]
       ; images which are used for baseline determination as there's no need to
       ; include them in the calculation
       ; the matrix is calculated from the aif with cut off second 'bumps'

    aifmatrix = fltarr( nshotsdt-(*bl)[1]-1, nshotsdt-(*bl)[1]-1 )
    for i = 0, nshotsdt-(*bl)[1]-2 do begin
       for j = 0, nshotsdt-(*bl)[1]-i-2 do begin
         aifmatrix[i+j, i] = aifc[j]
       end
    end

       ; define the result arrays for cbf, cbv, mtt

    cbf = fltarr( xsize, ysize )
    cbv = fltarr( xsize, ysize )
    mtt = fltarr( xsize, ysize )

       ; one residue

    R=fltarr( nshotsdt-(*bl)[1]-1 )

       ; now do the single value decomposition of the aif_matrix such that
       ; aifmatrix = U x W x V where U and V are orthonormal and W diagonal

    SVDC, aifmatrix, W, U, V, /double, /column

       ; for the diagonal matrix, remove the singular elements, i.e. those that have
       ; a value of less then 15% the maximum value

    wcut = matrixcutoff * max(abs(W))
    W    = (W ge wcut ) * W

		; now do the calculation for each image pixel for this slice
		; The loop itself consumes about 6 seconds of time on my machine (as compared
		; to 10 seconds for the svsol) - 3 seconds of which are needed to fill 'signal'.
		; It actually makes no difference whether signal is defined (signal[*] = fltarr(something))
		; or newly created in each loop - the latter is even a little bit faster.
		; another 1.5 seconds are spend on the s0[x,y] gt sigthres comparison

    s0     = *(*(baselineset->imgdata()))[slice]
    ctr    = (*(ctrset->imgdata()))[slice]

	signal = fltarr( nshotsdt-(*bl)[1]-1 )
	totaif = total(aif)
	for x = 0, xsize-1 do begin
		for y = 0, ysize-1 do begin
			signal[nshotsdt-nshots:*] = (*ctr)[x,y,(*bl)[1]+1:nshots-1]
			totsig = total(signal)
			if( s0[x,y] gt sigthres and totsig gt 0. ) then begin
				R[*] = svsol( u, w, v, signal, /double, /column )
				cbf[x,y] = 6000 * 0.705 / tr * max(R[*]) * totsig / ( totaif * total(R[*]) )
				cbv[x,y] = 0.705 * 100. * totsig / totaif
				mtt[x,y] = total(R[*]) * tr / max(R[*])
			end
		end
	end

       ; cut off extreme values

    cbv = cbv < (*(*self.g).preferences).ssvd_cbvcutoff
    cbv = cbv > 0.
    cbf = cbf < (*(*self.g).preferences).ssvd_cbfcutoff
    cbf = cbf > 0.
    mtt = mtt < (*(*self.g).preferences).ssvd_mttcutoff
    mtt = mtt > 0.

		; store the result in the resultsets

	ptr_free, (*(cbvset->imgdata()))[slice]
	ptr_free, (*(cbfset->imgdata()))[slice]
	ptr_free, (*(mttset->imgdata()))[slice]

	(*(cbvset->imgdata()))[slice] = ptr_new( cbv, /no_copy )
	(*(cbfset->imgdata()))[slice] = ptr_new( cbf, /no_copy )
	(*(mttset->imgdata()))[slice] = ptr_new( mtt, /no_copy )

end


;+=----------------------------------------------------------------------------
; Name  : mod_rsvd__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_rsvd__define
	obj = { MOD_RSVD		   			,$
			inherits LUPEMOD 	  		,$
			mdisp:			 obj_new()	,$
			disp1:			 obj_new()	,$
			disp2:			 obj_new()	,$
			disp3:			 obj_new()	,$
			progressbar:	 obj_new()	,$
			idcbvname:				0L	,$
			idcbfname:				0L	,$
			idmttname:				0L	,$
			idsigthres:				0L	,$
			idctrthres:				0L	,$
			idwcutoff:				0L	,$
			idcbvcutoff:			0L	,$
			idcbfcutoff:			0L	,$
			idmttcutoff:			0L	,$
			idtimeshift:			0L	 $
		  }
end

