;+=----------------------------------------------------------------------------
; Name    : mod_save
; RCS     : $Id: mod_save__define.pro,v 1.2 2004/11/25 09:09:08 olivert Exp olivert $
; Desc    : Saves images of class 'result' and 'color' in the formats DICOM3.0
;           and several bitmap formats.
;           To be able to save in Dicom format, the parent imageSet must be
;           present in memory.
; Bugs    : only when > 6 result images are available:
;           'set series numbers' will take the value of the first *displayed*
;           set as starting value and uses it on the first entry in the list
;           (which is inconsistent)
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_save::revision
	rev = '$Revision: 1.2 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_save::init, g

    res = self->lupemod::init( 'mod_save', 'embedded', 0, g )
    if( res eq 0 ) then return, 0

		; default settings for buttons

	self.file_format      = 0
	self.jpg_quality      = 100
	self.flag_anonymize   = 0
	self.flag_scale       = 1
	self.flag_resize      = 0
	self.resize_factor    = 1.
	self.ipmethod         = 1

		; create empty hash tables to start with

	self.hImgSets      = ptrarr(2)
	self.hBasenames    = objarr(2)
	self.hDescriptions = objarr(2)
	self.hSeriesNo     = objarr(2)
	for i = 0, 1 do begin
		self.hImgSets[i]      = ptr_new( strarr(1), /no_copy) ; dummy to start with
		self.hBasenames[i]    = obj_new('mrhash', 'string' )  ; name -> file basename
		self.hDescriptions[i] = obj_new('mrhash', 'string' )  ; name -> dicom description
		self.hSeriesNo[i]     = obj_new('mrhash', 'int'    )  ; name -> series number
	end

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_save::readyToExecute
; Desc  : We're always ready - if there's no result/color image sets available
;         the lists are empty - so what?
;=-----------------------------------------------------------------------------
function mod_save::readyToExecute, option
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_save::map
; Desc  : Use for 'top' and 'embedded' modules
;=-----------------------------------------------------------------------------
pro mod_save::map, option

	i18n = (*self.g).i18n

		; create the base widget

	res = self->createBaseWidget( /column )
	if( res eq 0 ) then return

		; and the rest...

	b  = widget_base( self.baseid, scr_ysize=50 )
	b  = widget_base( self.baseid, /row, scr_xsize=900, scr_ysize=470, xpad=60 )
	b2 = widget_base( b, /column, /frame )
	b1 = widget_base( b, /column, /frame, xpad=10, ypad=45  )

		; left column

	x  = widget_label( b2, value=i18n->get('save_title'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=210)

	c2 = widget_base ( b2, xsize=200, /align_center, /column )

	x  = widget_label( c2, value='', ysize=19 )

	x  = widget_label ( c2, value=i18n->get('save_format'), /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=25 )
	x  = widget_label ( r1, value=i18n->get('save_fileFormat'), xsize=100 )
	x  = widget_droplist( r1, value=['Dicom', 'JPEG', 'TIFF', 'PNG', 'BMP'], xsize=70, uvalue={o:self, m:'fileFormatSelectEvt'})
	widget_control, x, set_droplist_select=self.file_format
	r1 = mrwidget_numfield( r, value=self.jpg_quality, min=1, max=100, incr=5, labeltext=i18n->get('save_jpgQualityFactor'), /integer, /all_events, labelsize=120, fieldsize=30, uvalue={o:self, m:'jpgQualitySelectEvt'} )

	x  = widget_label( c2, value='', ysize=21 )

	x  = widget_label ( c2, value='Dicom:', /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=25 )
	x  = widget_label ( r1, value=i18n->get('save_anonymizeDicom'), xsize=150 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	x  = widget_button( r2, value='', uvalue={o:self, m:'anonymizeButtonEvt'} )
	widget_control, x, set_button=self.flag_anonymize
	r1 = widget_base  ( r , /row, ysize=30 )
	x  = widget_label ( r1, value=i18n->get('save_scaleToFullRange'), xsize=150 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	x  = widget_button( r2, value='', uvalue={o:self, m:'scaleButtonEvt'} )
	widget_control, x, set_button=self.flag_scale
	r1 = widget_base  ( r, /column, xsize=180 )
	x  = widget_button( r1, value=i18n->get('save_setConsecutiveNumbers'), uvalue={o:self, m:'setSeriesNoButtonEvt'} )

	x  = widget_label( c2, value='', ysize=21 )

	x  = widget_label ( c2, value=i18n->get('save_images'), /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=25 )
	x  = widget_label ( r1, value=i18n->get('save_resizeImages'), xsize=150 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	x  = widget_button( r2, value='', uvalue={o:self, m:'resizeButtonEvt'} )
	widget_control, x, set_button=self.flag_resize
	r1 = mrwidget_numfield( r, /editable, value=self.resize_factor, min=1, max=5, incr=.5, labeltext=i18n->get('save_resizeFactor'), /float, /all_events, labelsize=120, fieldsize=30, format='(f3.1)', uvalue={o:self, m:'resizeFactorSelectEvt'} )
	if( self.flag_resize eq 0 ) then widget_control, r1, sensitive=0
	self.idresizefactor = r1
	r1 = widget_base  ( r , /row, ysize=25 )
	self.idiplabel  = widget_label ( r1, value=i18n->get('save_interpolationMethod'), xsize=100 )
	self.idipmethod = widget_droplist( r1, value=[i18n->get('save_nearestNeighbour'),i18n->get('save_linear'),i18n->get('save_cubic')], xsize=70, uvalue={o:self, m:'ipMethodSelectEvt'})
	widget_control, self.idipmethod, set_droplist_select=self.ipmethod
	if( self.flag_resize eq 0 ) then begin
		widget_control, self.idiplabel, sensitive=0
		widget_control, self.idipmethod, sensitive=0
	end

	x  = widget_label( c2, value='', ysize=26 )
	x  = widget_button( c2, value=i18n->get('CloseModule'), xsize=186, /frame, uvalue={o:self, m:'closeButtonEvt'} )

		; right block - one for 'result', one for 'color'

	for i = 0, 1 do begin

		if( i eq 0 ) then begin
			x = widget_label( b1, value=i18n->get('save_resultImages'), /align_left )
		end else begin
			x = widget_label( b1, value='', scr_ysize=20 )
			x = widget_label( b1, value=i18n->get('save_colorImages'), /align_left )
		end

		c1 = widget_base( b1, /frame, /column )
		cols = widget_base( c1, column=13 )

		col1 = widget_base( cols, /column )
		col2 = widget_base( cols, /column, scr_xsize=20, /base_align_left, /align_left)
		x    = widget_base( cols, /column, scr_xsize=20 )
		col3 = widget_base( cols, /column, scr_xsize=130, /base_align_left )
		x    = widget_base( cols, /column, scr_xsize=10 )
		col4 = widget_base( cols, /column, /base_align_center )
		x    = widget_base( cols, /column, scr_xsize=10 )
		col5 = widget_base( cols, /column, /base_align_center )
		x    = widget_base( cols, /column, scr_xsize=20 )
		col6 = widget_base( cols, /column, scr_xsize=50, /base_align_center )
		x    = widget_base( cols, /column, scr_xsize=10 )
		col7 = widget_base( cols, /column, /base_align_center )

		x = widget_label( col2, scr_ysize=25, value=i18n->get('save_number') )
		x = widget_label( col3, scr_ysize=25, value=i18n->get('save_name') )
		x = widget_label( col4, scr_ysize=25, value=i18n->get('save_fileBasename') )
		x = widget_label( col5, scr_ysize=25, value=i18n->get('save_seriesDescription') )
		x = widget_label( col6, scr_ysize=25, value=i18n->get('save_seriesNumber') )
		x = widget_label( col7, scr_ysize=25, value=i18n->get('save_save') )

			; the vertical slider

		x = widget_label ( col1, value='', scr_ysize=24 )
		self.idslider[i] = widget_slider( col1, scr_ysize=125, /vertical, value=100, /suppress_value, uvalue={o:self, m:'sliderChangeEvt'} )

			; the 6 lines

		for j = 0, 5 do begin
			self.idno[i,j]       = widget_label( col2, value=strtrim(j,2), scr_ysize=20, scr_xsize=18 )
			self.idname[i,j]     = widget_label( col3, value='', scr_ysize=20, scr_xsize=130 )
			self.idbasename[i,j] = widget_text ( col4, value='', /editable, scr_ysize=20, uvalue={o:self, m:'basenameChangeEvt'}, /all_events )
			self.iddescr[i,j]    = widget_text ( col5, value='', /editable, scr_ysize=20, uvalue={o:self, m:'descriptionChangeEvt'}, /all_events )
			self.idseriesno[i,j] = widget_text ( col6, value='0', /editable, scr_ysize=20, scr_xsize=30, uvalue={o:self, m:'seriesnoChangeEvt'}, /all_events )
			self.idsave[i,j]     = widget_button( col7, value='save.bmp', /bitmap, ysize=20, uvalue={o:self, m:'saveButtonEvt'} )

		end
	end

		; realize the widget and register with the xmanager

	self->placeAndRegister

		; update the internal tables and display

	self->updateInternals
	self->display

		; register ourselves for changes in image sets

	(*self.g).imgSets->notify, self, 'imgSetChangeEvt', ['add','remove']

end

;+=----------------------------------------------------------------------------
; Name  : mod_save::unmap
; Args  : <b>killed</b>
;         If the user killed the window via Ctrl-F4 or the close handle (the
;         cross at the upper right), killed is set to '1'. In this case it
;         might be (I have not tested this) that the system's window manager
;         has already removed the window from screen.
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::unmap, _ref_extra=ex

	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

		; place module cleanup code here

	(*self.g).imgSets->notify, self, /remove

		; destroy the module - taken care off by the superclass

	self->lupemod::unmap, _strict_extra=ex
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::execute
; Desc  : Use for 'noui' modules - 'top' and 'embedded' modules use map and unmap
;=-----------------------------------------------------------------------------
pro mod_save::execute, label
end


;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_save::fileFormatSelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::fileFormatSelectEvt, evt
	self.file_format = evt.index
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::jpgQualitySelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::jpgQualitySelectEvt, evt
	self.jpg_quality = evt.value
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::anonymizeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::anonymizeButtonEvt, evt
	self.flag_anonymize = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::scaleButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::scaleButtonEvt, evt
	self.flag_scale = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::setSeriesNoButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::setSeriesNoButtonEvt, evt
	i18n = (*self.g).i18n
	widget_control, self.idSeriesNo[0,0], get_value=sn
	sn = fix(sn)
	if( sn eq 0 ) then begin
		x = dialog_message([i18n->get('save_setSeriesNumber1'),i18n->get('save_setSeriesNumber2')], /info, title=i18n->get('Info'))
		return
	end
	for i = 0, 1 do begin                        ; this is easier than expected :)
		if( self.hSeriesNo[i]->count() eq 0 ) then continue
		sets = self.hSeriesNo[i]->keys()
		for j = 0, n_elements(sets)-1 do begin
			self.hSeriesNo[i]->set, sets[j], sn
			sn = sn+1
		end
	end
	self->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::resizeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::resizeButtonEvt, evt
	self.flag_resize = evt.select
	if( self.flag_resize ) then begin
		widget_control, self.idresizefactor, /sensitive
		widget_control, self.idiplabel,      /sensitive
		widget_control, self.idipmethod,     /sensitive
	end else begin
		widget_control, self.idresizefactor, sensitive=0
		widget_control, self.idiplabel,      sensitive=0
		widget_control, self.idipmethod,     sensitive=0
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::resizeFactorSelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::resizeFactorSelectEvt, evt
	self.resize_factor = evt.value
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::ipMethodSelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::ipMethodSelectEvt, evt
	self.ipMethod = evt.index
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::sliderChangeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::sliderChangeEvt, evt
	self->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::basenameChangeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::basenameChangeEvt, evt
	for i = 0, 1 do begin
		for j = 0, 5 do begin
			if( evt.id eq self.idbasename[i,j] ) then begin
				widget_control, self.idslider[i], get_value=sliderval
				minmax = widget_info( self.idslider[i], /slider_min_max )
				name = (*self.hImgSets[i])[ minmax[1]-sliderval+j ]
				widget_control, evt.id, get_value=basename
				self.hBasenames[i]->set, name, basename
				return
			end
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::descriptionChangeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::descriptionChangeEvt, evt
	for i = 0, 1 do begin
		for j = 0, 5 do begin
			if( evt.id eq self.iddescr[i,j] ) then begin
				widget_control, self.idslider[i], get_value=sliderval
				minmax = widget_info( self.idslider[i], /slider_min_max )
				name = (*self.hImgSets[i])[ minmax[1]-sliderval+j ]
				widget_control, evt.id, get_value=description
				self.hDescriptions[i]->set, name, description
				return
			end
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::seriesnoChangeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::seriesnoChangeEvt, evt
	for i = 0, 1 do begin
		for j = 0, 5 do begin
			if( evt.id eq self.idSeriesNo[i,j] ) then begin
				widget_control, self.idslider[i], get_value=sliderval
				minmax = widget_info( self.idslider[i], /slider_min_max )
				name = (*self.hImgSets[i])[ minmax[1]-sliderval+j ]
				widget_control, evt.id, get_value=seriesno
				self.hSeriesNo[i]->set, name, fix(seriesno)
				return
			end
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::saveButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::saveButtonEvt, evt
	for i = 0, 1 do begin
		for j = 0, 5 do begin
			if( evt.id eq self.idSave[i,j] ) then begin
				case i of
					0: class='result'
					1: class='color'
				end
				widget_control, self.idslider[i], get_value=sliderval
				minmax = widget_info( self.idslider[i], /slider_min_max )
				name = (*self.hImgSets[i])[ minmax[1]-sliderval+j ]
				if( self.file_format eq 0 ) then self->saveDicom, class, name, i, j else self->saveBitmap, class, name, i, j
			end
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_save::closeButtonEvt, evt
	self->unmap
end


;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_save::imgSetChangeEvt
; Desc  : Callback from the central image set container
;=-----------------------------------------------------------------------------
pro mod_save::imgSetChangeEvt, sender, type
	self->updateInternals
	self->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_save::updateInternals
; Desc  : This method updates the internal associative arrays that keep track
;         of all user input. Since this function is used upon callback from
;         the central imageset container, existing values should be kept as
;         much as possible.
;=-----------------------------------------------------------------------------
pro mod_save::updateInternals

	i18n = (*self.g).i18n

	imgsets      = ptrarr(2)
	basenames    = objarr(2)
	descriptions = objarr(2)
	seriesno     = objarr(2)

		; create the new hash tables

	for i = 0, 1 do begin
		basenames[i]    = obj_new('mrhash', 'string' ) ; name -> file basename
		descriptions[i] = obj_new('mrhash', 'string' ) ; name -> dicom description
		seriesno[i]     = obj_new('mrhash', 'int'    ) ; name -> series number
	end

		; first loop for result images, second loop for color images

	for i = 0, 1 do begin
		case i of
			0: allsets = mrGetImageSet( (*self.g).imgSets, class='result' )
			1: allsets = mrGetImageSet( (*self.g).imgSets, class='color'  )
		end

		if( allsets[0] eq obj_new() ) then continue

		nsets  = n_elements(allsets)
		imgsets[i] = ptr_new( strarr(nsets), /no_copy )

			; fill hashes - try to use existing values, otherwise fill defaults

		for setno = 0, nsets-1 do begin
			name = allsets[setno]->name()
			(*imgsets[i])[setno] = name

			if( self.hBasenames[i]->exists(name) and self.hBasenames[i]->get( name ) ne '' ) then begin ; yes, I know
				basenames[i]->set, name, self.hBasenames[i]->get( name )
			end else begin
				basenames[i]->set, name, strjoin(strsplit( name, /extract ), '_') ; replace whitespace by _
			end

			if( self.hDescriptions[i]->exists(name) and self.hDescriptions[i]->get( name ) ne '' ) then begin
				descriptions[i]->set, name, self.hDescriptions[i]->get( name )
			end else begin
				descriptions[i]->set, name, name + i18n->get('save_defaultDescription') + (*self.g).version
			end

			if( self.hSeriesNo[i]->exists(name) ) then begin
				seriesno[i]->set, name, self.hSeriesNo[i]->get( name )
			end else begin
				seriesno[i]->set, name, 0
			end
		end
	end

		; finally destroy the old (hash) lists and save the new ones

	for i = 0, 1 do begin
		ptr_free, self.hImgSets[i]
		obj_destroy, self.hBasenames[i]
		obj_destroy, self.hDescriptions[i]
		obj_destroy, self.hSeriesNo[i]
		self.hImgSets[i]      = imgSets[i]
		self.hBasenames[i]    = basenames[i]
		self.hDescriptions[i] = descriptions[i]
		self.hSeriesNo[i]     = seriesno[i]
	end

end

;+=----------------------------------------------------------------------------
; Name  : mod_save::display
; Desc  : Fill the values in the two blocks to the right. This takes the
;         current slider settings into account (and corrects them, if necessary)
;         i.e. after calls to updateInternals and display, everything on screen
;         should be alright.
;=-----------------------------------------------------------------------------
pro mod_save::display

		; we do this twice: once for the upper, once for the lower box

	for i = 0, 1 do begin
		if( self.hImgSets[i] eq ptr_new() ) then nsets=0 else nsets = n_elements( *self.hImgSets[i] )

			; take care of sliders and correct setting, if necessary

		if( nsets le 6 ) then begin
			widget_control, self.idslider[i], sensitive=0, set_slider_max=1, set_slider_min=0, set_value=1
			startID = 0
		end else begin
			widget_control, self.idslider[i], get_value=sliderval
			sliderval = sliderval < (nsets-6)
			startID   = nsets-6-sliderval
			widget_control, self.idslider[i], sensitive=1, set_slider_max=nsets-6, set_slider_min=0, set_value=sliderval
		end

			; display values

		for j = 0, 5 do begin
			widget_control, self.idno[i,j], set_value=strtrim(startId+j+1,2)
			if( startID+j ge nsets ) then begin
				widget_control, self.idname[i,j],     set_value=''
				widget_control, self.idbasename[i,j], set_value='', sensitive=0
				widget_control, self.iddescr[i,j],    set_value='', sensitive=0
				widget_control, self.idseriesno[i,j], set_value='', sensitive=0
				widget_control, self.idsave[i,j],     sensitive=0
			end else begin
				name = (*self.hImgSets[i])[startID + j]
				widget_control, self.idname[i,j],     set_value=name
				widget_control, self.idbasename[i,j], set_value=self.hBasenames[i]->get( name ), /sensitive
				widget_control, self.iddescr[i,j],    set_value=self.hDescriptions[i]->get( name ), /sensitive
				widget_control, self.idseriesno[i,j], set_value=strtrim(self.hSeriesNo[i]->get( name ),2), /sensitive
				widget_control, self.idsave[i,j],     /sensitive
			end
		end

	end
end


;+=----------------------------------------------------------------------------
; Name   : mod_save::saveDicom
; Desc   : Save result image sets in dicom format
;          set is 0 for result images and 1 for color images
;          idx is the index in the respective list
;          name is the name of the image set
;=-----------------------------------------------------------------------------
pro mod_save::saveDicom, class, name, set, idx

	i18n = (*self.g).i18n

	imgset = mrGetImageSet( (*self.g).imgSets, class=class, name=name )
	if( imgset eq obj_new() ) then begin                 ; shouldn't happen
		x = dialog_message(i18n->get('save_noSuchImageSetInMemory'), title=i18n->get('Error'), /error)
		return
	end

		; check for a valid series number

	seriesno = fix(self.hSeriesno[set]->get( name ))
	if( seriesno eq 0 ) then begin
		x = dialog_message([i18n->get('save_errSeriesNoGt0_1'), i18n->get('save_errSeriesNoGt0_2')], title=i18n->get('Error'), /error )
		return
	end

		; get the directory

	targetdir = dialog_pickfile( title=i18n->get('save_selectDirectory'), path=(*self.g).workingdir, /directory )
	if( targetdir eq '' ) then return        ; user pressed cancel and we silently return
	(*self.g).workingdir = targetdir

		; find the DICOM source - now a small helper function that
		; walks through the ancestry

	srcSet = findDicomParent( imgSet )

		; yes, even without a parent we could still write dicom format,
		; but it wouldn't make a lot of sense (read: I don't have the time
		; to fill in all the values manually. After all it's the user's own fault.
		; Yet another example for arrogant software :)

	if( srcSet eq obj_new() ) then begin
		x = dialog_message([i18n->get('save_noDicomSourceFound1'),i18n->get('save_noDicomSourceFound2'),i18n->get('save_noDicomSourceFound3')], title=i18n->get('Error'), /error)
		return
   	end

		; create one temporary mrdicom object which we'll use for saving all images

	dicom     = obj_new('mrdicom')
	imgData   = imgset->imgData()
	npics     = n_elements(*imgData)

		; create an array of mrDicom object references which will later be used as source
		; when copying the header data. As the source data might be of different format
		; (100s of signal images or just nslices, if derived from a loaded result set), we
		; need to do some checks here.

	if( imgSet->nSlices() ne srcSet->nSlices() ) then begin
		x = dialog_message([i18n->get('save_troubleWithParent'),i18n->get('save_pleaseContactAuthor')], title=i18n->get('Error'), /error)
		return
	end
	case srcSet->size(/n_dimensions) of
		1: srcData = *(srcSet->dicomData())
		2: begin
			srcData = objarr( srcSet->nSlices() )
			for i = 0, srcSet->nSlices()-1 do begin
				srcData[i] = (*(srcSet->dicomData()))[i,0]
			end
		end
		else: begin
			x = dialog_message([i18n->get('save_messedUpSourceSet'),i18n->get('save_pleaseContactAuthor')], title=i18n->get('Error'), /error )
			return
		end
	end

		; if scaling is switched on, we need to find the maximum image value for normalization
		; Obviously only b/w image values can be scaled

	if( self.flag_scale and set eq 0 ) then begin
		imgMaxval = 0
		for i = 0, npics-1 do imgMaxVal = max( *(*imgData)[i] ) > imgMaxVal ; jaja, we don't check here for valid pointers
	end

		; saving time... errgh, I mean, time for saving
		; as we don't know whether the user has changed the seriesnumber,
		; we always create a new seriesUID - that's not 100% clean, but 99.9%
		; (do it the Microsoft way: the user *must* have meant us to create a new seriesUID)
		; The studyUID is kept (copied with all header info)
		; Image UIDs are automatically created while writing

	seriesUID = dicom->generateUID('series')
	pb = obj_new('mrProgressbar', /modal, group_leader=self.baseid, title=i18n->get('save_savingProgress'), /floating)

	for i = 0, npics-1 do begin
		if( (*imgData)[i] eq ptr_new() ) then continue                     ; might happen, though it shouldn't
		img = *(*imgData)[i]

			; scaling result data, if checkbox ist set (only for result images)

		if( self.flag_scale and set eq 0 ) then img = img * 4095. / imgMaxVal

			; resize the image

		dims  = size( img, /dimensions )
		if( self.flag_resize ) then begin
			newdims = fix( dims * self.resize_factor )
			case self.ipmethod of
				0: img = congrid( temporary(img), newdims[0], newdims[1], /center )
				1: img = 0 > congrid( temporary(img), newdims[0], newdims[1], /center, /interp )
				2: img = 0 > congrid( temporary(img), newdims[0], newdims[1],  cubic=-.5 )
			end
		end

			; b/w (=result) images: convert image data to pointer
			; otherwise create the true color image

		if( set eq 0 ) then begin                     ; black/white images
			img = ptr_new( img, /no_copy )
		end else begin                                ; color images
			img = img < 255                           ; remove possible congrid effect
			img = ptr_new( img, /no_copy )
			colimg = createTrueColorImg( img, palette=imgset->palette() )
			ptr_free, img
			img    = colimg
		end


			; assemble filename

		if( i+1 lt 10 ) then ext='_s0' else ext='_s'
		filename = targetdir + self.hBasenames[set]->get(name) + ext + strtrim(i+1,2) + '.dcm'

			; prepare dicom object

		dicom->copy, srcData[i]

		dicom->setTagValue, 'rel_series_instance_uid', seriesUID
		dicom->setTagValue, 'rel_series_number',       seriesno
		dicom->setTagValue, 'id_series_description',   self.hDescriptions[set]->get(name)
		dicom->setTagValue, 'id_series_time',          dicom->getTagValue('id_image_time')
		dicom->setTagValue, 'id_series_date',          dicom->getTagValue('id_image_date')
		dicom->setTagValue, 'rel_image_number',        i+1
		dicom->setTagValue, 'img_rows',                dims[0]
		dicom->setTagValue, 'img_columns',             dims[1]
		dicom->setImg, img, /no_copy

		if( self.flag_anonymize ) then dicom->anonymize

		res = dicom->write( filename, flip=1 )

		ptr_free, img
		pb->update, (1.+i)/npics
	end
	obj_destroy, pb
end

;+=----------------------------------------------------------------------------
; Name   : mod_save::saveBitmap
; Desc   : Save result image sets in bitmap formats
;          set is 0 for result images and 1 for color images
;          idx is the index in the respective list
;          name is the name of the image set
;=-----------------------------------------------------------------------------
pro mod_save::saveBitmap, class, name, set, idx

	i18n = (*self.g).i18n

	imgset = mrGetImageSet( (*self.g).imgSets, class=class, name=name )

	targetdir = dialog_pickfile( title=i18n->get('save_selectDirectory'), path=(*self.g).workingdir, /directory )
	if( targetdir eq '' ) then return        ; user pressed cancel and we silently return
	(*self.g).workingdir = targetdir

	imgData   = imgset->imgData()
	npics     = n_elements(*imgData)

		; for some of the formats, we need to scale the result image

	imgMaxval = 0
	for i = 0, npics-1 do imgMaxVal = max( *(*imgData)[i] ) > imgMaxVal ; jaja, we don't check here for valid pointers

		; a progressbar

	pb = obj_new('mrProgressbar', /modal, group_leader=self.baseid, title=i18n->get('save_savingProgress'), /floating)

		; loop over all images and save

	for i = 0, npics-1 do begin

		if( i+1 lt 10 ) then ext='_s0' else ext='_s'
		filename = targetdir + self.hBasenames[set]->get(name) + ext + strtrim(i+1,2)

			; resize the image

		img = *(*imgData)[i]
		dims  = size( img, /dimensions )
		if( self.flag_resize ) then begin
			newdims = fix( dims * self.resize_factor )
			case self.ipmethod of
				0: img = congrid( temporary(img), newdims[0], newdims[1], /center )
				1: img = 0 > congrid( temporary(img), newdims[0], newdims[1], /center, /interp )
				2: img = 0 > congrid( temporary(img), newdims[0], newdims[1],  cubic=-.5 )
			end
		end

			; here's the file format specific stuff (which includes writing the file)

		case self.file_format of
		1: begin ; JPG
			if( set eq 0 ) then begin
				img = 0 > fix(255.* img/imgMaxval) < 255
				write_jpeg, filename+'.jpg', img, quality=self.jpg_quality
			end else begin
				img = ptr_new( img, /no_copy )
				colimg = createTrueColorImg( img, palette=imgset->palette() )
				ptr_free, img
				write_jpeg, filename+'.jpg', *colimg, quality=self.jpg_quality, true=1
				ptr_free, colimg
			end
		end
		2: begin ; TIFF
			if( set eq 0 ) then begin
				img = 0 > fix(255.* img/imgMaxval) < 255
				write_tiff, filename+'.tif', img ; /float didn't work :-(
			end else begin
				(imgset->palette())->getProperty, red_values=r, green_values=g, blue_values=b
				img = rotate(img,7)
				write_tiff, filename+'.tif', img, red=r, green=g, blue=b
			end
		end
		3: begin ; PNG
			if( set eq 0 ) then begin
				img = 0 > fix(255.* img/imgMaxval) < 255
				rgb = indgen( 256 )
				write_png, filename+'.png', img, rgb, rgb, rgb
			end else begin
				(imgset->palette())->getProperty, red_values=r, green_values=g, blue_values=b
				write_png, filename+'.png', img, r, g, b
			end
		end
		4: begin ; BMP
			if( set eq 0 ) then begin
				img = 0 > fix(255.* img/imgMaxval) < 255
				rgb = indgen( 256 )
				write_bmp, filename+'.bmp', img, rgb, rgb, rgb
			end else begin
				(imgset->palette())->getProperty, red_values=r, green_values=g, blue_values=b
				write_bmp, filename+'.bmp', img, r, g, b
			end
		end
		end
		pb->update, (1.+i)/npics
	end
	obj_destroy, pb
end

;+=----------------------------------------------------------------------------
; Name  : mod_save__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_save__define
	obj = { MOD_SAVE		   		,$
			inherits LUPEMOD   		,$
			idno:				lonarr(2,6)	,$ ;
			idname:				lonarr(2,6) ,$ ;
			idbasename:			lonarr(2,6) ,$ ;
			iddescr:			lonarr(2,6) ,$ ;
			idseriesno:			lonarr(2,6) ,$ ;
			idsave:				lonarr(2,6) ,$ ;
			idslider:			lonarr(2)   ,$ ;
			idjpgquality:				0L	,$ ;
			idresizefactor:				0L	,$ ;
			idiplabel:					0L	,$ ;
			idipmethod:					0L	,$ ;
			file_format:				0	,$ ;
			jpg_quality:				0	,$ ;
			flag_anonymize:				0	,$ ;
			flag_scale:					0	,$ ;
			flag_resize:				0	,$ ;
			resize_factor:				1.	,$ ;
			ipmethod:					0	,$ ;
			hImgSets:			ptrarr(2)	,$ ; names of the image sets
			hBasenames:			objarr(2)	,$ ; the hash tables which store the user given
			hDescriptions:		objarr(2)	,$ ; values
			hSeriesNo:			objarr(2)	 $ ;
		  }
end
